/*
 * This file is generated by gdbus-codegen, do not modify it.
 *
 * The license of this code is the same as for the D-Bus interface description
 * it was derived from. Note that it links to GLib, so must comply with the
 * LGPL linking clauses.
 */

#ifndef __IIO_SENSOR_PROXY_DBUS_H__
#define __IIO_SENSOR_PROXY_DBUS_H__

#include <gio/gio.h>

G_BEGIN_DECLS


/* ------------------------------------------------------------------------ */
/* Declarations for net.hadess.SensorProxy */

#define MS_DBUS_TYPE_SENSOR_PROXY (ms_dbus_sensor_proxy_get_type ())
#define MS_DBUS_SENSOR_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), MS_DBUS_TYPE_SENSOR_PROXY, MsDBusSensorProxy))
#define MS_DBUS_IS_SENSOR_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), MS_DBUS_TYPE_SENSOR_PROXY))
#define MS_DBUS_SENSOR_PROXY_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), MS_DBUS_TYPE_SENSOR_PROXY, MsDBusSensorProxyIface))

struct _MsDBusSensorProxy;
typedef struct _MsDBusSensorProxy MsDBusSensorProxy;
typedef struct _MsDBusSensorProxyIface MsDBusSensorProxyIface;

struct _MsDBusSensorProxyIface
{
  GTypeInterface parent_iface;


  gboolean (*handle_claim_accelerometer) (
    MsDBusSensorProxy *object,
    GDBusMethodInvocation *invocation);

  gboolean (*handle_claim_light) (
    MsDBusSensorProxy *object,
    GDBusMethodInvocation *invocation);

  gboolean (*handle_claim_proximity) (
    MsDBusSensorProxy *object,
    GDBusMethodInvocation *invocation);

  gboolean (*handle_release_accelerometer) (
    MsDBusSensorProxy *object,
    GDBusMethodInvocation *invocation);

  gboolean (*handle_release_light) (
    MsDBusSensorProxy *object,
    GDBusMethodInvocation *invocation);

  gboolean (*handle_release_proximity) (
    MsDBusSensorProxy *object,
    GDBusMethodInvocation *invocation);

  const gchar * (*get_accelerometer_orientation) (MsDBusSensorProxy *object);

  gboolean  (*get_has_accelerometer) (MsDBusSensorProxy *object);

  gboolean  (*get_has_ambient_light) (MsDBusSensorProxy *object);

  gboolean  (*get_has_proximity) (MsDBusSensorProxy *object);

  gdouble  (*get_light_level) (MsDBusSensorProxy *object);

  const gchar * (*get_light_level_unit) (MsDBusSensorProxy *object);

  gboolean  (*get_proximity_near) (MsDBusSensorProxy *object);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (MsDBusSensorProxy, g_object_unref)
#endif

GType ms_dbus_sensor_proxy_get_type (void) G_GNUC_CONST;

GDBusInterfaceInfo *ms_dbus_sensor_proxy_interface_info (void);
guint ms_dbus_sensor_proxy_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void ms_dbus_sensor_proxy_complete_claim_accelerometer (
    MsDBusSensorProxy *object,
    GDBusMethodInvocation *invocation);

void ms_dbus_sensor_proxy_complete_release_accelerometer (
    MsDBusSensorProxy *object,
    GDBusMethodInvocation *invocation);

void ms_dbus_sensor_proxy_complete_claim_light (
    MsDBusSensorProxy *object,
    GDBusMethodInvocation *invocation);

void ms_dbus_sensor_proxy_complete_release_light (
    MsDBusSensorProxy *object,
    GDBusMethodInvocation *invocation);

void ms_dbus_sensor_proxy_complete_claim_proximity (
    MsDBusSensorProxy *object,
    GDBusMethodInvocation *invocation);

void ms_dbus_sensor_proxy_complete_release_proximity (
    MsDBusSensorProxy *object,
    GDBusMethodInvocation *invocation);



/* D-Bus method calls: */
void ms_dbus_sensor_proxy_call_claim_accelerometer (
    MsDBusSensorProxy *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean ms_dbus_sensor_proxy_call_claim_accelerometer_finish (
    MsDBusSensorProxy *proxy,
    GAsyncResult *res,
    GError **error);

gboolean ms_dbus_sensor_proxy_call_claim_accelerometer_sync (
    MsDBusSensorProxy *proxy,
    GCancellable *cancellable,
    GError **error);

void ms_dbus_sensor_proxy_call_release_accelerometer (
    MsDBusSensorProxy *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean ms_dbus_sensor_proxy_call_release_accelerometer_finish (
    MsDBusSensorProxy *proxy,
    GAsyncResult *res,
    GError **error);

gboolean ms_dbus_sensor_proxy_call_release_accelerometer_sync (
    MsDBusSensorProxy *proxy,
    GCancellable *cancellable,
    GError **error);

void ms_dbus_sensor_proxy_call_claim_light (
    MsDBusSensorProxy *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean ms_dbus_sensor_proxy_call_claim_light_finish (
    MsDBusSensorProxy *proxy,
    GAsyncResult *res,
    GError **error);

gboolean ms_dbus_sensor_proxy_call_claim_light_sync (
    MsDBusSensorProxy *proxy,
    GCancellable *cancellable,
    GError **error);

void ms_dbus_sensor_proxy_call_release_light (
    MsDBusSensorProxy *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean ms_dbus_sensor_proxy_call_release_light_finish (
    MsDBusSensorProxy *proxy,
    GAsyncResult *res,
    GError **error);

gboolean ms_dbus_sensor_proxy_call_release_light_sync (
    MsDBusSensorProxy *proxy,
    GCancellable *cancellable,
    GError **error);

void ms_dbus_sensor_proxy_call_claim_proximity (
    MsDBusSensorProxy *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean ms_dbus_sensor_proxy_call_claim_proximity_finish (
    MsDBusSensorProxy *proxy,
    GAsyncResult *res,
    GError **error);

gboolean ms_dbus_sensor_proxy_call_claim_proximity_sync (
    MsDBusSensorProxy *proxy,
    GCancellable *cancellable,
    GError **error);

void ms_dbus_sensor_proxy_call_release_proximity (
    MsDBusSensorProxy *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean ms_dbus_sensor_proxy_call_release_proximity_finish (
    MsDBusSensorProxy *proxy,
    GAsyncResult *res,
    GError **error);

gboolean ms_dbus_sensor_proxy_call_release_proximity_sync (
    MsDBusSensorProxy *proxy,
    GCancellable *cancellable,
    GError **error);



/* D-Bus property accessors: */
gboolean ms_dbus_sensor_proxy_get_has_accelerometer (MsDBusSensorProxy *object);
void ms_dbus_sensor_proxy_set_has_accelerometer (MsDBusSensorProxy *object, gboolean value);

const gchar *ms_dbus_sensor_proxy_get_accelerometer_orientation (MsDBusSensorProxy *object);
gchar *ms_dbus_sensor_proxy_dup_accelerometer_orientation (MsDBusSensorProxy *object);
void ms_dbus_sensor_proxy_set_accelerometer_orientation (MsDBusSensorProxy *object, const gchar *value);

gboolean ms_dbus_sensor_proxy_get_has_ambient_light (MsDBusSensorProxy *object);
void ms_dbus_sensor_proxy_set_has_ambient_light (MsDBusSensorProxy *object, gboolean value);

const gchar *ms_dbus_sensor_proxy_get_light_level_unit (MsDBusSensorProxy *object);
gchar *ms_dbus_sensor_proxy_dup_light_level_unit (MsDBusSensorProxy *object);
void ms_dbus_sensor_proxy_set_light_level_unit (MsDBusSensorProxy *object, const gchar *value);

gdouble ms_dbus_sensor_proxy_get_light_level (MsDBusSensorProxy *object);
void ms_dbus_sensor_proxy_set_light_level (MsDBusSensorProxy *object, gdouble value);

gboolean ms_dbus_sensor_proxy_get_has_proximity (MsDBusSensorProxy *object);
void ms_dbus_sensor_proxy_set_has_proximity (MsDBusSensorProxy *object, gboolean value);

gboolean ms_dbus_sensor_proxy_get_proximity_near (MsDBusSensorProxy *object);
void ms_dbus_sensor_proxy_set_proximity_near (MsDBusSensorProxy *object, gboolean value);


/* ---- */

#define MS_DBUS_TYPE_SENSOR_PROXY_PROXY (ms_dbus_sensor_proxy_proxy_get_type ())
#define MS_DBUS_SENSOR_PROXY_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), MS_DBUS_TYPE_SENSOR_PROXY_PROXY, MsDBusSensorProxyProxy))
#define MS_DBUS_SENSOR_PROXY_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), MS_DBUS_TYPE_SENSOR_PROXY_PROXY, MsDBusSensorProxyProxyClass))
#define MS_DBUS_SENSOR_PROXY_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), MS_DBUS_TYPE_SENSOR_PROXY_PROXY, MsDBusSensorProxyProxyClass))
#define MS_DBUS_IS_SENSOR_PROXY_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), MS_DBUS_TYPE_SENSOR_PROXY_PROXY))
#define MS_DBUS_IS_SENSOR_PROXY_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), MS_DBUS_TYPE_SENSOR_PROXY_PROXY))

typedef struct _MsDBusSensorProxyProxy MsDBusSensorProxyProxy;
typedef struct _MsDBusSensorProxyProxyClass MsDBusSensorProxyProxyClass;
typedef struct _MsDBusSensorProxyProxyPrivate MsDBusSensorProxyProxyPrivate;

struct _MsDBusSensorProxyProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  MsDBusSensorProxyProxyPrivate *priv;
};

struct _MsDBusSensorProxyProxyClass
{
  GDBusProxyClass parent_class;
};

GType ms_dbus_sensor_proxy_proxy_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (MsDBusSensorProxyProxy, g_object_unref)
#endif

void ms_dbus_sensor_proxy_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
MsDBusSensorProxy *ms_dbus_sensor_proxy_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
MsDBusSensorProxy *ms_dbus_sensor_proxy_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void ms_dbus_sensor_proxy_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
MsDBusSensorProxy *ms_dbus_sensor_proxy_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
MsDBusSensorProxy *ms_dbus_sensor_proxy_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define MS_DBUS_TYPE_SENSOR_PROXY_SKELETON (ms_dbus_sensor_proxy_skeleton_get_type ())
#define MS_DBUS_SENSOR_PROXY_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), MS_DBUS_TYPE_SENSOR_PROXY_SKELETON, MsDBusSensorProxySkeleton))
#define MS_DBUS_SENSOR_PROXY_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), MS_DBUS_TYPE_SENSOR_PROXY_SKELETON, MsDBusSensorProxySkeletonClass))
#define MS_DBUS_SENSOR_PROXY_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), MS_DBUS_TYPE_SENSOR_PROXY_SKELETON, MsDBusSensorProxySkeletonClass))
#define MS_DBUS_IS_SENSOR_PROXY_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), MS_DBUS_TYPE_SENSOR_PROXY_SKELETON))
#define MS_DBUS_IS_SENSOR_PROXY_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), MS_DBUS_TYPE_SENSOR_PROXY_SKELETON))

typedef struct _MsDBusSensorProxySkeleton MsDBusSensorProxySkeleton;
typedef struct _MsDBusSensorProxySkeletonClass MsDBusSensorProxySkeletonClass;
typedef struct _MsDBusSensorProxySkeletonPrivate MsDBusSensorProxySkeletonPrivate;

struct _MsDBusSensorProxySkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  MsDBusSensorProxySkeletonPrivate *priv;
};

struct _MsDBusSensorProxySkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType ms_dbus_sensor_proxy_skeleton_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (MsDBusSensorProxySkeleton, g_object_unref)
#endif

MsDBusSensorProxy *ms_dbus_sensor_proxy_skeleton_new (void);


/* ------------------------------------------------------------------------ */
/* Declarations for net.hadess.SensorProxy.Compass */

#define MS_DBUS_TYPE_SENSOR_PROXY_COMPASS (ms_dbus_sensor_proxy_compass_get_type ())
#define MS_DBUS_SENSOR_PROXY_COMPASS(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), MS_DBUS_TYPE_SENSOR_PROXY_COMPASS, MsDBusSensorProxyCompass))
#define MS_DBUS_IS_SENSOR_PROXY_COMPASS(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), MS_DBUS_TYPE_SENSOR_PROXY_COMPASS))
#define MS_DBUS_SENSOR_PROXY_COMPASS_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), MS_DBUS_TYPE_SENSOR_PROXY_COMPASS, MsDBusSensorProxyCompassIface))

struct _MsDBusSensorProxyCompass;
typedef struct _MsDBusSensorProxyCompass MsDBusSensorProxyCompass;
typedef struct _MsDBusSensorProxyCompassIface MsDBusSensorProxyCompassIface;

struct _MsDBusSensorProxyCompassIface
{
  GTypeInterface parent_iface;


  gboolean (*handle_claim_compass) (
    MsDBusSensorProxyCompass *object,
    GDBusMethodInvocation *invocation);

  gboolean (*handle_release_compass) (
    MsDBusSensorProxyCompass *object,
    GDBusMethodInvocation *invocation);

  gdouble  (*get_compass_heading) (MsDBusSensorProxyCompass *object);

  gboolean  (*get_has_compass) (MsDBusSensorProxyCompass *object);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (MsDBusSensorProxyCompass, g_object_unref)
#endif

GType ms_dbus_sensor_proxy_compass_get_type (void) G_GNUC_CONST;

GDBusInterfaceInfo *ms_dbus_sensor_proxy_compass_interface_info (void);
guint ms_dbus_sensor_proxy_compass_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void ms_dbus_sensor_proxy_compass_complete_claim_compass (
    MsDBusSensorProxyCompass *object,
    GDBusMethodInvocation *invocation);

void ms_dbus_sensor_proxy_compass_complete_release_compass (
    MsDBusSensorProxyCompass *object,
    GDBusMethodInvocation *invocation);



/* D-Bus method calls: */
void ms_dbus_sensor_proxy_compass_call_claim_compass (
    MsDBusSensorProxyCompass *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean ms_dbus_sensor_proxy_compass_call_claim_compass_finish (
    MsDBusSensorProxyCompass *proxy,
    GAsyncResult *res,
    GError **error);

gboolean ms_dbus_sensor_proxy_compass_call_claim_compass_sync (
    MsDBusSensorProxyCompass *proxy,
    GCancellable *cancellable,
    GError **error);

void ms_dbus_sensor_proxy_compass_call_release_compass (
    MsDBusSensorProxyCompass *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean ms_dbus_sensor_proxy_compass_call_release_compass_finish (
    MsDBusSensorProxyCompass *proxy,
    GAsyncResult *res,
    GError **error);

gboolean ms_dbus_sensor_proxy_compass_call_release_compass_sync (
    MsDBusSensorProxyCompass *proxy,
    GCancellable *cancellable,
    GError **error);



/* D-Bus property accessors: */
gboolean ms_dbus_sensor_proxy_compass_get_has_compass (MsDBusSensorProxyCompass *object);
void ms_dbus_sensor_proxy_compass_set_has_compass (MsDBusSensorProxyCompass *object, gboolean value);

gdouble ms_dbus_sensor_proxy_compass_get_compass_heading (MsDBusSensorProxyCompass *object);
void ms_dbus_sensor_proxy_compass_set_compass_heading (MsDBusSensorProxyCompass *object, gdouble value);


/* ---- */

#define MS_DBUS_TYPE_SENSOR_PROXY_COMPASS_PROXY (ms_dbus_sensor_proxy_compass_proxy_get_type ())
#define MS_DBUS_SENSOR_PROXY_COMPASS_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), MS_DBUS_TYPE_SENSOR_PROXY_COMPASS_PROXY, MsDBusSensorProxyCompassProxy))
#define MS_DBUS_SENSOR_PROXY_COMPASS_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), MS_DBUS_TYPE_SENSOR_PROXY_COMPASS_PROXY, MsDBusSensorProxyCompassProxyClass))
#define MS_DBUS_SENSOR_PROXY_COMPASS_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), MS_DBUS_TYPE_SENSOR_PROXY_COMPASS_PROXY, MsDBusSensorProxyCompassProxyClass))
#define MS_DBUS_IS_SENSOR_PROXY_COMPASS_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), MS_DBUS_TYPE_SENSOR_PROXY_COMPASS_PROXY))
#define MS_DBUS_IS_SENSOR_PROXY_COMPASS_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), MS_DBUS_TYPE_SENSOR_PROXY_COMPASS_PROXY))

typedef struct _MsDBusSensorProxyCompassProxy MsDBusSensorProxyCompassProxy;
typedef struct _MsDBusSensorProxyCompassProxyClass MsDBusSensorProxyCompassProxyClass;
typedef struct _MsDBusSensorProxyCompassProxyPrivate MsDBusSensorProxyCompassProxyPrivate;

struct _MsDBusSensorProxyCompassProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  MsDBusSensorProxyCompassProxyPrivate *priv;
};

struct _MsDBusSensorProxyCompassProxyClass
{
  GDBusProxyClass parent_class;
};

GType ms_dbus_sensor_proxy_compass_proxy_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (MsDBusSensorProxyCompassProxy, g_object_unref)
#endif

void ms_dbus_sensor_proxy_compass_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
MsDBusSensorProxyCompass *ms_dbus_sensor_proxy_compass_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
MsDBusSensorProxyCompass *ms_dbus_sensor_proxy_compass_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void ms_dbus_sensor_proxy_compass_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
MsDBusSensorProxyCompass *ms_dbus_sensor_proxy_compass_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
MsDBusSensorProxyCompass *ms_dbus_sensor_proxy_compass_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define MS_DBUS_TYPE_SENSOR_PROXY_COMPASS_SKELETON (ms_dbus_sensor_proxy_compass_skeleton_get_type ())
#define MS_DBUS_SENSOR_PROXY_COMPASS_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), MS_DBUS_TYPE_SENSOR_PROXY_COMPASS_SKELETON, MsDBusSensorProxyCompassSkeleton))
#define MS_DBUS_SENSOR_PROXY_COMPASS_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), MS_DBUS_TYPE_SENSOR_PROXY_COMPASS_SKELETON, MsDBusSensorProxyCompassSkeletonClass))
#define MS_DBUS_SENSOR_PROXY_COMPASS_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), MS_DBUS_TYPE_SENSOR_PROXY_COMPASS_SKELETON, MsDBusSensorProxyCompassSkeletonClass))
#define MS_DBUS_IS_SENSOR_PROXY_COMPASS_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), MS_DBUS_TYPE_SENSOR_PROXY_COMPASS_SKELETON))
#define MS_DBUS_IS_SENSOR_PROXY_COMPASS_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), MS_DBUS_TYPE_SENSOR_PROXY_COMPASS_SKELETON))

typedef struct _MsDBusSensorProxyCompassSkeleton MsDBusSensorProxyCompassSkeleton;
typedef struct _MsDBusSensorProxyCompassSkeletonClass MsDBusSensorProxyCompassSkeletonClass;
typedef struct _MsDBusSensorProxyCompassSkeletonPrivate MsDBusSensorProxyCompassSkeletonPrivate;

struct _MsDBusSensorProxyCompassSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  MsDBusSensorProxyCompassSkeletonPrivate *priv;
};

struct _MsDBusSensorProxyCompassSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType ms_dbus_sensor_proxy_compass_skeleton_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (MsDBusSensorProxyCompassSkeleton, g_object_unref)
#endif

MsDBusSensorProxyCompass *ms_dbus_sensor_proxy_compass_skeleton_new (void);


G_END_DECLS

#endif /* __IIO_SENSOR_PROXY_DBUS_H__ */
