
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#pragma once

#include <glib-object.h>

G_BEGIN_DECLS

/* enumerations from "ms-enums.h" */

GType ms_feedback_profile_get_type (void);
#define MS_TYPE_FEEDBACK_PROFILE (ms_feedback_profile_get_type())

GType ms_phosh_notification_urgency_get_type (void);
#define MS_TYPE_PHOSH_NOTIFICATION_URGENCY (ms_phosh_notification_urgency_get_type())

GType ms_media_role_get_type (void);
#define MS_TYPE_MEDIA_ROLE (ms_media_role_get_type())

G_END_DECLS

/* Generated data ends here */

