#ifndef __RESOURCE_mobile_settings_H__
#define __RESOURCE_mobile_settings_H__

#include <gio/gio.h>

G_GNUC_INTERNAL GResource *mobile_settings_get_resource (void);

G_GNUC_INTERNAL void mobile_settings_register_resource (void);
G_GNUC_INTERNAL void mobile_settings_unregister_resource (void);

#endif
