/*
 * This file is generated by gdbus-codegen, do not modify it.
 *
 * The license of this code is the same as for the D-Bus interface description
 * it was derived from. Note that it links to GLib, so must comply with the
 * LGPL linking clauses.
 */

#ifndef __LOGIN1_MANAGER_DBUS_H__
#define __LOGIN1_MANAGER_DBUS_H__

#include <gio/gio.h>

G_BEGIN_DECLS


/* ------------------------------------------------------------------------ */
/* Declarations for org.freedesktop.login1.Manager */

#define MS_PLUGIN_LIBREM5_DBUS_TYPE_LOGIN_MANAGER (ms_plugin_librem5_dbus_login_manager_get_type ())
#define MS_PLUGIN_LIBREM5_DBUS_LOGIN_MANAGER(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), MS_PLUGIN_LIBREM5_DBUS_TYPE_LOGIN_MANAGER, MsPluginLibrem5DBusLoginManager))
#define MS_PLUGIN_LIBREM5_DBUS_IS_LOGIN_MANAGER(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), MS_PLUGIN_LIBREM5_DBUS_TYPE_LOGIN_MANAGER))
#define MS_PLUGIN_LIBREM5_DBUS_LOGIN_MANAGER_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), MS_PLUGIN_LIBREM5_DBUS_TYPE_LOGIN_MANAGER, MsPluginLibrem5DBusLoginManagerIface))

struct _MsPluginLibrem5DBusLoginManager;
typedef struct _MsPluginLibrem5DBusLoginManager MsPluginLibrem5DBusLoginManager;
typedef struct _MsPluginLibrem5DBusLoginManagerIface MsPluginLibrem5DBusLoginManagerIface;

struct _MsPluginLibrem5DBusLoginManagerIface
{
  GTypeInterface parent_iface;

  gboolean (*handle_can_suspend) (
    MsPluginLibrem5DBusLoginManager *object,
    GDBusMethodInvocation *invocation);

  gboolean (*handle_suspend) (
    MsPluginLibrem5DBusLoginManager *object,
    GDBusMethodInvocation *invocation,
    gboolean arg_unnamed_arg0);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (MsPluginLibrem5DBusLoginManager, g_object_unref)
#endif

GType ms_plugin_librem5_dbus_login_manager_get_type (void) G_GNUC_CONST;

GDBusInterfaceInfo *ms_plugin_librem5_dbus_login_manager_interface_info (void);
guint ms_plugin_librem5_dbus_login_manager_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void ms_plugin_librem5_dbus_login_manager_complete_suspend (
    MsPluginLibrem5DBusLoginManager *object,
    GDBusMethodInvocation *invocation);

void ms_plugin_librem5_dbus_login_manager_complete_can_suspend (
    MsPluginLibrem5DBusLoginManager *object,
    GDBusMethodInvocation *invocation,
    const gchar *unnamed_arg0);



/* D-Bus method calls: */
void ms_plugin_librem5_dbus_login_manager_call_suspend (
    MsPluginLibrem5DBusLoginManager *proxy,
    gboolean arg_unnamed_arg0,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean ms_plugin_librem5_dbus_login_manager_call_suspend_finish (
    MsPluginLibrem5DBusLoginManager *proxy,
    GAsyncResult *res,
    GError **error);

gboolean ms_plugin_librem5_dbus_login_manager_call_suspend_sync (
    MsPluginLibrem5DBusLoginManager *proxy,
    gboolean arg_unnamed_arg0,
    GCancellable *cancellable,
    GError **error);

void ms_plugin_librem5_dbus_login_manager_call_can_suspend (
    MsPluginLibrem5DBusLoginManager *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean ms_plugin_librem5_dbus_login_manager_call_can_suspend_finish (
    MsPluginLibrem5DBusLoginManager *proxy,
    gchar ** out_unnamed_arg0,
    GAsyncResult *res,
    GError **error);

gboolean ms_plugin_librem5_dbus_login_manager_call_can_suspend_sync (
    MsPluginLibrem5DBusLoginManager *proxy,
    gchar ** out_unnamed_arg0,
    GCancellable *cancellable,
    GError **error);



/* ---- */

#define MS_PLUGIN_LIBREM5_DBUS_TYPE_LOGIN_MANAGER_PROXY (ms_plugin_librem5_dbus_login_manager_proxy_get_type ())
#define MS_PLUGIN_LIBREM5_DBUS_LOGIN_MANAGER_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), MS_PLUGIN_LIBREM5_DBUS_TYPE_LOGIN_MANAGER_PROXY, MsPluginLibrem5DBusLoginManagerProxy))
#define MS_PLUGIN_LIBREM5_DBUS_LOGIN_MANAGER_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), MS_PLUGIN_LIBREM5_DBUS_TYPE_LOGIN_MANAGER_PROXY, MsPluginLibrem5DBusLoginManagerProxyClass))
#define MS_PLUGIN_LIBREM5_DBUS_LOGIN_MANAGER_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), MS_PLUGIN_LIBREM5_DBUS_TYPE_LOGIN_MANAGER_PROXY, MsPluginLibrem5DBusLoginManagerProxyClass))
#define MS_PLUGIN_LIBREM5_DBUS_IS_LOGIN_MANAGER_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), MS_PLUGIN_LIBREM5_DBUS_TYPE_LOGIN_MANAGER_PROXY))
#define MS_PLUGIN_LIBREM5_DBUS_IS_LOGIN_MANAGER_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), MS_PLUGIN_LIBREM5_DBUS_TYPE_LOGIN_MANAGER_PROXY))

typedef struct _MsPluginLibrem5DBusLoginManagerProxy MsPluginLibrem5DBusLoginManagerProxy;
typedef struct _MsPluginLibrem5DBusLoginManagerProxyClass MsPluginLibrem5DBusLoginManagerProxyClass;
typedef struct _MsPluginLibrem5DBusLoginManagerProxyPrivate MsPluginLibrem5DBusLoginManagerProxyPrivate;

struct _MsPluginLibrem5DBusLoginManagerProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  MsPluginLibrem5DBusLoginManagerProxyPrivate *priv;
};

struct _MsPluginLibrem5DBusLoginManagerProxyClass
{
  GDBusProxyClass parent_class;
};

GType ms_plugin_librem5_dbus_login_manager_proxy_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (MsPluginLibrem5DBusLoginManagerProxy, g_object_unref)
#endif

void ms_plugin_librem5_dbus_login_manager_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
MsPluginLibrem5DBusLoginManager *ms_plugin_librem5_dbus_login_manager_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
MsPluginLibrem5DBusLoginManager *ms_plugin_librem5_dbus_login_manager_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void ms_plugin_librem5_dbus_login_manager_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
MsPluginLibrem5DBusLoginManager *ms_plugin_librem5_dbus_login_manager_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
MsPluginLibrem5DBusLoginManager *ms_plugin_librem5_dbus_login_manager_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define MS_PLUGIN_LIBREM5_DBUS_TYPE_LOGIN_MANAGER_SKELETON (ms_plugin_librem5_dbus_login_manager_skeleton_get_type ())
#define MS_PLUGIN_LIBREM5_DBUS_LOGIN_MANAGER_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), MS_PLUGIN_LIBREM5_DBUS_TYPE_LOGIN_MANAGER_SKELETON, MsPluginLibrem5DBusLoginManagerSkeleton))
#define MS_PLUGIN_LIBREM5_DBUS_LOGIN_MANAGER_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), MS_PLUGIN_LIBREM5_DBUS_TYPE_LOGIN_MANAGER_SKELETON, MsPluginLibrem5DBusLoginManagerSkeletonClass))
#define MS_PLUGIN_LIBREM5_DBUS_LOGIN_MANAGER_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), MS_PLUGIN_LIBREM5_DBUS_TYPE_LOGIN_MANAGER_SKELETON, MsPluginLibrem5DBusLoginManagerSkeletonClass))
#define MS_PLUGIN_LIBREM5_DBUS_IS_LOGIN_MANAGER_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), MS_PLUGIN_LIBREM5_DBUS_TYPE_LOGIN_MANAGER_SKELETON))
#define MS_PLUGIN_LIBREM5_DBUS_IS_LOGIN_MANAGER_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), MS_PLUGIN_LIBREM5_DBUS_TYPE_LOGIN_MANAGER_SKELETON))

typedef struct _MsPluginLibrem5DBusLoginManagerSkeleton MsPluginLibrem5DBusLoginManagerSkeleton;
typedef struct _MsPluginLibrem5DBusLoginManagerSkeletonClass MsPluginLibrem5DBusLoginManagerSkeletonClass;
typedef struct _MsPluginLibrem5DBusLoginManagerSkeletonPrivate MsPluginLibrem5DBusLoginManagerSkeletonPrivate;

struct _MsPluginLibrem5DBusLoginManagerSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  MsPluginLibrem5DBusLoginManagerSkeletonPrivate *priv;
};

struct _MsPluginLibrem5DBusLoginManagerSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType ms_plugin_librem5_dbus_login_manager_skeleton_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (MsPluginLibrem5DBusLoginManagerSkeleton, g_object_unref)
#endif

MsPluginLibrem5DBusLoginManager *ms_plugin_librem5_dbus_login_manager_skeleton_new (void);


G_END_DECLS

#endif /* __LOGIN1_MANAGER_DBUS_H__ */
