
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#include "ms-enum-types.h"
#include "ms-enums.h"

#define C_ENUM(v) ((gint) v)
#define C_FLAGS(v) ((guint) v)

/* enumerations from "ms-enums.h" */

GType
ms_feedback_profile_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (MS_FEEDBACK_PROFILE_FULL), "MS_FEEDBACK_PROFILE_FULL", "full" },
        { C_ENUM (MS_FEEDBACK_PROFILE_QUIET), "MS_FEEDBACK_PROFILE_QUIET", "quiet" },
        { C_ENUM (MS_FEEDBACK_PROFILE_SILENT), "MS_FEEDBACK_PROFILE_SILENT", "silent" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("MsFeedbackProfile"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
ms_phosh_notification_urgency_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (MS_PHOSH_NOTIFICATION_NONE), "MS_PHOSH_NOTIFICATION_NONE", "none" },
        { C_ENUM (MS_PHOSH_NOTIFICATION_URGENCY_LOW), "MS_PHOSH_NOTIFICATION_URGENCY_LOW", "urgency-low" },
        { C_ENUM (MS_PHOSH_NOTIFICATION_URGENCY_NORMAL), "MS_PHOSH_NOTIFICATION_URGENCY_NORMAL", "urgency-normal" },
        { C_ENUM (MS_PHOSH_NOTIFICATION_URGENCY_CRITICAL), "MS_PHOSH_NOTIFICATION_URGENCY_CRITICAL", "urgency-critical" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("MsPhoshNotificationUrgency"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
ms_media_role_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (MS_MEDIA_ROLE_ALARM), "MS_MEDIA_ROLE_ALARM", "alarm" },
        { C_ENUM (MS_MEDIA_ROLE_ALERT), "MS_MEDIA_ROLE_ALERT", "alert" },
        { C_ENUM (MS_MEDIA_ROLE_MULTIMEDIA), "MS_MEDIA_ROLE_MULTIMEDIA", "multimedia" },
        { C_ENUM (MS_MEDIA_ROLE_NOTIFICATION), "MS_MEDIA_ROLE_NOTIFICATION", "notification" },
        { C_ENUM (MS_MEDIA_ROLE_PHONE), "MS_MEDIA_ROLE_PHONE", "phone" },
        { C_ENUM (MS_MEDIA_ROLE_RINGTONE), "MS_MEDIA_ROLE_RINGTONE", "ringtone" },
        { C_ENUM (MS_MEDIA_ROLE_DEFAULT), "MS_MEDIA_ROLE_DEFAULT", "default" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("MsMediaRole"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* Generated data ends here */

